//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//
//
// Use of this source code is subject to the terms of the Microsoft end-user
// license agreement (EULA) under which you licensed this SOFTWARE PRODUCT.
// If you did not accept the terms of the EULA, you are not authorized to use
// this source code. For a copy of the EULA, please see the LICENSE.RTF on your
// install media.
//
//------------------------------------------------------------------------------
//
//  File:  init.c
//
//  Samsung SMDK2440A board initialization code.
//
#include <bsp.h>


static void InitDisplay();

extern DWORD CEProcessorType;
void ConfigureGPIO(void);

#ifdef DVS_EN
//-------------------------------------------
// Variables for DVS
static int CurrVoltage;
extern volatile int CurrentState;
extern volatile int PrevState;
extern volatile int NextState;

//---------------------------------------------------------------------------
// Function for DVS
void ChangeVoltage(int);
int GetCurrentVoltage(void);

// ----------------------------------------------------------------------------
#endif

UINT32 g_oalIoCtlClockSpeed;
//------------------------------------------------------------------------------
//
//  Function:  OEMInit
//
//  This is Windows CE OAL initialization function. It is called from kernel
//  after basic initialization is made.
//
void OEMInit()
{

#ifdef DVS_EN
	volatile S3C2440A_IOPORT_REG *s2440IOP = (S3C2440A_IOPORT_REG *)OALPAtoVA(S3C2440A_BASE_REG_PA_IOPORT, FALSE);
#if (DVS_METHOD == 3)
	volatile S3C2440A_PWM_REG *g_pPWMRegs = (S3C2440A_PWM_REG*)OALPAtoUA(S3C2440A_BASE_REG_PA_PWM);
    	UINT32 sysIntr = SYSINTR_NOP;
    	//UINT32 irq;
	//UINT32 tcon;
#endif
#endif
	NKForceCleanBoot();

    OALMSG(OAL_FUNC, (L"+OEMInit\r\n"));

	g_oalIoCtlClockSpeed = S3C2440A_FCLK;
	
    CEProcessorType=PROCESSOR_STRONGARM;

    // Set memory size for DrWatson kernel support
    dwNKDrWatsonSize = 128 * 1024;

    // Initilize cache globals
    OALCacheGlobalsInit();

    OALLogSerial(
        L"DCache: %d sets, %d ways, %d line size, %d size\r\n", 
        g_oalCacheInfo.L1DSetsPerWay, g_oalCacheInfo.L1DNumWays,
        g_oalCacheInfo.L1DLineSize, g_oalCacheInfo.L1DSize
    );
    OALLogSerial(
        L"ICache: %d sets, %d ways, %d line size, %d size\r\n", 
        g_oalCacheInfo.L1ISetsPerWay, g_oalCacheInfo.L1INumWays,
        g_oalCacheInfo.L1ILineSize, g_oalCacheInfo.L1ISize
    );
    
    // Initialize interrupts
    if (!OALIntrInit()) {
        OALMSG(OAL_ERROR, (
            L"ERROR: OEMInit: failed to initialize interrupts\r\n"
        ));
    }

    // Initialize system clock
	OALTimerInit(RESCHED_PERIOD, OEM_COUNT_1MS, 0);

	ConfigureGPIO();

	InitDisplay();

    // Initialize the KITL connection if required
    OALKitlStart();

    OALMSG(OAL_FUNC, (L"-OEMInit\r\n"));
#ifdef DVS_EN
#if (Eval_Probe == 1)
	s2440IOP->MISCCR = (s2440IOP->MISCCR & ~(0x7 << 8)) | (0x3 << 8);		// CLKSEL1,     011 -> HCLK
	s2440IOP->MISCCR = (s2440IOP->MISCCR & ~(0x7 << 4)) | (0x4 << 4);		// CLKSEL0,     100 -> PCLK
	s2440IOP->GPHCON = (s2440IOP->GPHCON & ~(0x3 << 18)) | (0x2 << 18);		// GPH9 -> CLKOUT0
	s2440IOP->GPHCON = (s2440IOP->GPHCON & ~(0x3 << 20)) | (0x2 << 20);		// GPH10 -> CLKOUT1
#endif //(Eval_Probe == 1)	

#if (DVS_METHOD == 3)
	CurrentState = Active;
#endif
#endif
}
//------------------------------------------------------------------------------

# include "StartLogo.c" 

static void InitDisplay(DWORD LcdType)
{
    volatile S3C2440A_IOPORT_REG *s2440IOP = (S3C2440A_IOPORT_REG *)OALPAtoVA(S3C2440A_BASE_REG_PA_IOPORT, FALSE);
    volatile S3C2440A_LCD_REG    *s2440LCD = (S3C2440A_LCD_REG *)OALPAtoVA(S3C2440A_BASE_REG_PA_LCD, FALSE);
	unsigned int clkval_calc;  // VCLKƵ.
    DWORD XSize, YSize;
    DWORD VBPD, VFPD, VSPW; // źŵǰʱ,Լ.
    DWORD HBPD, HFPD, HSPW; // 

    // Set up the LCD controller registers to display a power-on bitmap image.
    //
    s2440IOP->GPCUP     = 0xFFFFFFFF;
    s2440IOP->GPCCON    = 0xAAAAAAAA;
                                    
	//s2440IOP->GPEDAT    = 0x1;
    s2440IOP->GPDUP     = 0xFFFFFFFF;
    s2440IOP->GPDCON    = 0xAAAAAAAA; //0xAAAAAAA1; 
    /*
    * GPG4 is lcd power enable pin.set GPG4 high.
    */
    s2440IOP->GPGCON   &= ~(3 << 8);
    s2440IOP->GPGCON   |=  (1 << 8);
    s2440IOP->GPGDAT   |=  (1 << 4);
    

	 //Temp by Russell
    clkval_calc = LCD_PIXCLOCK;
    XSize = LCD_WIDTH;
    YSize = LCD_HEIGHT;
    VBPD  =	LCD_UPPER_MARGIN;
    VFPD  =	LCD_LOWER_MARGIN;
    VSPW  = LCD_VSYNC_LEN;
    HBPD  = LCD_RIGHT_MARGIN;
    HFPD  =	LCD_LEFT_MARGIN;
    HSPW  =	LCD_HSYNC_LEN;


		s2440LCD->LCDCON1   =  (LCD_PIXCLOCK <<  8) |       /* VCLK = HCLK / ((CLKVAL + 1) * 2) -> About 7 Mhz  */
                           (0            <<  7) |       /*(LCD_MVAL    <<  7)  |    */   /* 0 : Each Frame                                   */
                           (3            <<  5) |       /* TFT LCD Pannel                                   */
                           (12           <<  1) |       /* 16bpp Mode                                       */
                           (0            <<  0) ;       /* Disable LCD Output                               */

   	s2440LCD->LCDCON2   =  (VBPD        << 24) |	// VBPD          :   1
	    				   ((YSize-1)   << 14) |	// Virtical Size : 320 - 1
  						   (VFPD        <<  6) |	// VFPD          :   2
   						   (VSPW        <<  0) ;	// VSPW          :   1

   	s2440LCD->LCDCON3   =  (HBPD        << 19) |	// HBPD          :   6
   						   ((XSize-1)   <<  8) |	// HOZVAL_TFT    : 240 - 1
   						   (HFPD        <<  0) ;	// HFPD          :   2

   	s2440LCD->LCDCON4   =  (13          <<  8) |	// MVAL          :  13                              */
   						   (LCD_HSYNC_LEN        <<  0) ;	// HSPW          :   4                              */

#if !defined(LCD_CON5)
#    define LCD_CON5 ((1<<11) | (1 << 9) | (1 << 8) | (1 << 3) | (1 << 0))
#endif

    s2440LCD->LCDCON5   =  LCD_CON5;

    s2440LCD->LCDSADDR1 = ((IMAGE_FRAMEBUFFER_DMA_BASE >> 22)     << 21) | 
                          ((M5D(IMAGE_FRAMEBUFFER_DMA_BASE >> 1)) <<  0);

    s2440LCD->LCDSADDR2 = M5D((IMAGE_FRAMEBUFFER_DMA_BASE + (XSize * YSize * 2)) >> 1);

    s2440LCD->LCDSADDR3 = (((XSize - XSize) / 1) << 11) | (XSize / 1);        

    //s2440LCD->TCONSEL   |= 0x3;
    s2440LCD->LCDINTMSK |= 3;
  	s2440LCD->TCONSEL   &= (~7);
    //    s2440LCD->TCONSEL   |= (0x1<<4);

   	s2440LCD->TPAL     = 0x0; 
   	s2440LCD->TCONSEL &= ~((1<<4) | 1);								// Disable LCC3600, LCP3600
   	s2440IOP->GPGUP	   = s2440IOP->GPGUP  & (~(1<<4)) | (1<<4);		// Pull-up disbale
   	s2440IOP->GPGCON   = s2440IOP->GPGCON & (~(3<<8)) | (3<<8);
        
   	s2440LCD->LCDCON5  = s2440LCD->LCDCON5 & (~(1<<3)) | (1<<3);	// PWREN
   	s2440LCD->LCDCON5  = s2440LCD->LCDCON5 & (~(1<<5)) | (0<<5);	// INVPWREN    
    s2440LCD->LCDCON1  |= 1;

    // Display a bitmap image on the LCD...
    //
    memset((void *)IMAGE_FRAMEBUFFER_UA_BASE, 0, XSize*YSize*2);
	{
		unsigned PicWidth   = StartLogoData[0];
		unsigned PicHeight  = StartLogoData[1];
		unsigned CopyWidth  = PicWidth  <= LCD_WIDTH  ? PicWidth  : LCD_WIDTH ;
		unsigned CopyHeight = PicHeight <= LCD_HEIGHT ? PicHeight : LCD_HEIGHT;

		unsigned V;
		const unsigned short *PicPtr = StartLogoData + 2;
		unsigned short *LCDPtr = (unsigned short *)IMAGE_FRAMEBUFFER_UA_BASE; 
		for (V = 0; V < CopyHeight; V++) {
			memcpy(LCDPtr, PicPtr, CopyWidth * 2);
			LCDPtr += LCD_WIDTH;
			PicPtr += PicWidth;
		}
	}



#ifdef DVS_EN
#if (DVS_METHOD == 1 || DVS_METHOD == 3)
    	// Enable LCD Interrupt for DVS + CLOCK Change.
    	s2440LCD->LCDINTMSK = (1); // UnMASK LCD Frame Sub Interrupt
    	//s2440INT->INTMSK &= ~(1 << IRQ_LCD);
    	//RETAILMSG(1,(TEXT("-E5-\r\n")));
#endif
#endif



}


void ConfigureGPIO()
{
    volatile S3C2440A_IOPORT_REG *s2440IOP = (S3C2440A_IOPORT_REG *)OALPAtoVA(S3C2440A_BASE_REG_PA_IOPORT, FALSE);
    volatile S3C2440A_CLKPWR_REG *s2440CLKPWR = (S3C2440A_CLKPWR_REG *)OALPAtoVA(S3C2440A_BASE_REG_PA_CLOCK_POWER, FALSE);

    s2440IOP->GPACON    = 0x7fffff;

    s2440IOP->GPBDAT    = 0x62;
    s2440IOP->GPBUP     = 0x7FF;
    s2440IOP->GPBCON    = 0x15401; //0x2A96A4;

    s2440IOP->GPCUP     = 0xFFFF;
    s2440IOP->GPCCON    = 0xAAAAAAAA;
                                    
	s2440IOP->GPDDAT    = 0x1;
    s2440IOP->GPDUP     = 0xFFFF;
    s2440IOP->GPDCON    = 0xAAAAAAA1;
    
    s2440IOP->GPEDAT    = 0x0;
    s2440IOP->GPEUP     = 0xFFFF;
    s2440IOP->GPECON    = 0xAAAAAAAA;
    
    s2440IOP->GPECON    = (s2440IOP->GPECON & ~(0x3<<30)) | (0x1<<30);
    s2440IOP->GPECON    = (s2440IOP->GPECON & ~(0x3<<28)) | (0x1<<28);
    //s2440IOP->GPFDAT    = 0xF0;
    //s2440IOP->GPFUP     = 0xFF;
    //s2440IOP->GPFCON    = 0x55aa;
    
    s2440IOP->GPGDAT    = 0x1000;
    s2440IOP->GPGUP     = 0xFEFF;
    s2440IOP->GPGCON    = 0x0480FFBA;
    s2440IOP->GPGDAT    |= 0x1<<12;
    s2440IOP->GPGDAT    &= ~(0x1U<<12);
    s2440IOP->GPGUP     |= (0x1U<<12);
    s2440IOP->GPGCON    = (s2440IOP->GPGCON & ~(0x3<<24)) | (0x1<<24);

    s2440IOP->GPHDAT    = 0x0;
    s2440IOP->GPHUP     = 0x7FF;
    s2440IOP->GPHCON    = 0x14AAAA;

    s2440IOP->GPJDAT    = 0x1000;
    s2440IOP->GPJUP     = 0x1FFF;
    s2440IOP->GPJCON    = 0x1AAAAA;

	s2440IOP->MISCCR &= ~(7<<20);
	s2440IOP->MISCCR |= (4<<20);
	s2440IOP->MISCCR &= ~(7<<8);
	s2440IOP->MISCCR |= (5<<8);
	s2440IOP->MISCCR &= ~(7<<4);
	s2440IOP->MISCCR |= (5<<4);
	s2440IOP->MISCCR |= (3<<0);
	
}

#ifdef DVS_EN
void ChangeVoltage(int vtg)
{
	volatile S3C2440A_IOPORT_REG *s2440IOP = (S3C2440A_IOPORT_REG *)OALPAtoVA(S3C2440A_BASE_REG_PA_IOPORT, FALSE);
	UINT8 temp;
	
//  port setting  
// GPF4:D0, GPF5:D1, GPF6:D2, GPF7:D3, GPB7:D4, GPB8:Latch enable
	CurrVoltage = vtg;
	temp = s2440IOP->GPFDAT;
	switch(vtg) {
	case V090:	// 0.9V
		s2440IOP->GPBDAT=(s2440IOP->GPBDAT&0x77f)|(1<<7);   //D4
		s2440IOP->GPFDAT=(s2440IOP->GPFDAT&0x0f)|(0<<7)|(0<<6)|(1<<5)|(1<<4);   //D3~0
		break;
	case V095:	// 0.95V
		s2440IOP->GPBDAT=(s2440IOP->GPBDAT&0x77f)|(1<<7);   //D4
		s2440IOP->GPFDAT=(s2440IOP->GPFDAT&0x0f)|(0<<7)|(0<<6)|(0<<5)|(1<<4);   //D3~0
		break;
	case V0975:	// 0.975V
		s2440IOP->GPBDAT=(s2440IOP->GPBDAT&0x77f)|(1<<7);   //D4
		s2440IOP->GPFDAT=(s2440IOP->GPFDAT&0x0f)|(0<<7)|(0<<6)|(0<<5)|(0<<4);   //D3~0
		break;
	case V100:	// 1.0V
		s2440IOP->GPBDAT=(s2440IOP->GPBDAT&0x77f)|(0<<7);   //D4
		s2440IOP->GPFDAT=(s2440IOP->GPFDAT&0x0f)|(1<<7)|(1<<6)|(1<<5)|(1<<4);   //D3~0
		break;
	case V105:	// 1.05V
		s2440IOP->GPBDAT=(s2440IOP->GPBDAT&0x77f)|(0<<7);   //D4
		s2440IOP->GPFDAT=(s2440IOP->GPFDAT&0x0f)|(1<<7)|(1<<6)|(1<<5)|(0<<4);   //D3~0
		break;
	case V110:	// 1.1V
		s2440IOP->GPBDAT=(s2440IOP->GPBDAT&0x77f)|(0<<7);   //D4
		s2440IOP->GPFDAT=(s2440IOP->GPFDAT&0x0f)|(1<<7)|(1<<6)|(0<<5)|(1<<4);   //D3~0
		break;
	case V115:	// 1.15V
		s2440IOP->GPBDAT=(s2440IOP->GPBDAT&0x77f)|(0<<7);   //D4
		s2440IOP->GPFDAT=(s2440IOP->GPFDAT&0x0f)|(1<<7)|(1<<6)|(0<<5)|(0<<4);   //D3~0
		break;
	case V120:	// 1.2V
		s2440IOP->GPBDAT=(s2440IOP->GPBDAT&0x77f)|(0<<7);   //D4
		s2440IOP->GPFDAT=(s2440IOP->GPFDAT&0x0f)|(1<<7)|(0<<6)|(1<<5)|(1<<4);   //D3~0
		break;
	case V125:	// 1.25V
		s2440IOP->GPBDAT=(s2440IOP->GPBDAT&0x77f)|(0<<7);   //D4
		s2440IOP->GPFDAT=(s2440IOP->GPFDAT&0x0f)|(1<<7)|(0<<6)|(1<<5)|(0<<4);   //D3~0
		break;
	case V130:	// 1.3V
		s2440IOP->GPBDAT=(s2440IOP->GPBDAT&0x77f)|(0<<7);   //D4
		s2440IOP->GPFDAT=(s2440IOP->GPFDAT&0x0f)|(1<<7)|(0<<6)|(0<<5)|(1<<4);   //D3~0
		break;
	case V135:	// 1.35V
		s2440IOP->GPBDAT=(s2440IOP->GPBDAT&0x77f)|(0<<7);   //D4
		s2440IOP->GPFDAT=(s2440IOP->GPFDAT&0x0f)|(1<<7)|(0<<6)|(0<<5)|(0<<4);   //D3~0
		break;
	case V140:	// 1.4V
		s2440IOP->GPBDAT=(s2440IOP->GPBDAT&0x77f)|(0<<7);   //D4
		s2440IOP->GPFDAT=(s2440IOP->GPFDAT&0x0f)|(0<<7)|(1<<6)|(1<<5)|(1<<4);   //D3~0
		break;
	case V145:	// 1.45V
		s2440IOP->GPBDAT=(s2440IOP->GPBDAT&0x77f)|(0<<7);   //D4
		s2440IOP->GPFDAT=(s2440IOP->GPFDAT&0x0f)|(0<<7)|(1<<6)|(1<<5)|(0<<4);   //D3~0
		break;
	case V150:	// 1.5V
		s2440IOP->GPBDAT=(s2440IOP->GPBDAT&0x77f)|(0<<7);   //D4
		s2440IOP->GPFDAT=(s2440IOP->GPFDAT&0x0f)|(0<<7)|(1<<6)|(0<<5)|(1<<4);   //D3~0
		break;

	default:
		break;
	}

//  port setting  
// GPF4:D0, GPF5:D1, GPF6:D2, GPF7:D3, GPB7:D4, GPB8:Latch enable, GPB10: OE
	s2440IOP->GPBCON=(s2440IOP->GPBCON&0x3f3fff)|(1<<14);   // GPB7: Output
//	s2440IOP->rGPBUP&=~(1<<7);
	s2440IOP->GPFCON=(s2440IOP->GPFCON&0x00ff)|(0x5500);  // GPF4~7: Output 
//	s2440IOP->rGPFUP&=~(0xf<<4);

	s2440IOP->GPBDAT&=~(1<<8);   //Latch enable
	s2440IOP->GPBCON=(s2440IOP->GPBCON&0x3cffff)|(1<<16);   // GPB8: Output

	s2440IOP->GPBDAT|=(1<<10);   //Output enable
	s2440IOP->GPBCON=(s2440IOP->GPBCON&0x0fffff)|(1<<20);   // GPB10: Output
	
//	s2440IOP->rGPBUP&=~(1<<8);
	s2440IOP->GPBDAT|=(1<<8);   //Latch disable
	
	s2440IOP->GPFDAT = temp;
}

int GetCurrentVoltage(void)
{
	return CurrVoltage;
}
#endif
